import 'package:flutter/material.dart';
import 'package:flutter_bluetooth_serial/flutter_bluetooth_serial.dart';//引用藍牙模組

class BluetoothDevices extends StatelessWidget {
  const BluetoothDevices({super.key});

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Bluetooth Devices'),                         //藍牙搜尋標頭
      ),
      body: FutureBuilder(
        future: FlutterBluetoothSerial.instance.getBondedDevices(),     //取得裝置
        builder: (context, snapshot) {
          if (snapshot.hasData) {                                       //FutureBuilder 收聽stream找到資料
            List<BluetoothDevice> devices = snapshot.requireData;       //建置清單
            if (devices.isNotEmpty) {                                   //存在裝置
              return ListView.builder(
                itemCount: devices.length,                              //找到裝置數量
                itemBuilder: (context, index) {                         //放置找到裝置內容
                  return ListTile(                                      //列出清單
                    title: Text(devices[index].name ?? 'Unknown device'), //顯示裝置名噌稱，或未知裝置
                    subtitle: Text(devices[index].address),
                    onTap: () {                                        //點擊
                      Navigator.of(context).pop(devices[index]);       //點擊後返回主視窗
                    },
                  );
                },
              );
            }
            else {
              return const Center(
                child: Text('No devices found'),                        //搜尋不到任何裝置
              );
            }
          } else {
            return const Center(
              child: LinearProgressIndicator(),                        //進度調顯示
            );
          }
        },
      ),
    );
  }
}
